//
// Patron pour la generation d'une classe derivee dans ROOT
// FEventProcessor derive de TNamed
//

#ifndef FEventProcessor_h
#define FEventProcessor_h

#include "TNamed.h"
#include "TH1.h"
#include "FParam.h"
#include "FHistAtt.h"
#include "TString.h"
#include "THashList.h"
#include "FDataReaderWithDevice.h"
#include "FNetHistoProvider.h"
#include "TROOT.h"

class FEventProcessor : public TNamed {
  public:
    static void Make(Char_t *nom,Bool_t thread=kFALSE);
  
  protected:
    FDataReader     *dataReader; // Lecture des donnees
    THashList   *listOfRawParams; // liste des parametres bruts
    THashList   *listOfUserDefParams; // liste des parametres ajoues par l'utilisateur
    THashList 	*listOfParams; // liste de tous les parametres
    THashList 	*listOfRateCounters; // liste des FRateCounter
    THashList 	*listOfHistos; // liste des hitogrammes

    THashList 	*listOfHitParams; // liste des parametres touches
    THashList 	*listOfHistToFill; // liste des histogrammes a remplir
    
    TList	*listOfLines; // liste des lignes du fichier de configuration

    FNetHistoProvider *fnhp;

    Long_t	lsent;
    Bool_t	doSaveHistos;

    Bool_t  stop;

    ULong64_t nevt;

  protected:
    virtual void Init(void);       // initialisation
    virtual void AddRawParam(FParam *p);
    virtual void AddUserDefParam(FParam *p);
    virtual void AddRateCounterParam(FParam *p);

  public:
    FEventProcessor(void);    // constructeur par defaut
    FEventProcessor(const FEventProcessor &ap); // constructeur par copie

    ~FEventProcessor(void);		// desctructeur;

#if ROOT_VERSION_CODE >= ROOT_VERSION(3,4,0)
    virtual void Copy(TObject & obj) const;
#else
    virtual void Copy(TObject & obj);
#endif
    FEventProcessor& operator = (const FEventProcessor &hp); // operateur =

    virtual void Reset(void);
    virtual void Run(void);
    virtual void ProcessEvent(void);
    virtual void MakeListHistToFill(void);
    virtual void FillHistos(void);
    virtual void InitParams(void);
    virtual void InitHistos(void)=0;
    virtual Bool_t IsRunning(void){return !stop;}
    virtual void SetParamValues(void);
    virtual void IncrementCounters(void);
    virtual void SaveHistos(void);
    virtual void SaveConfig(Char_t const *cname="Config");
    virtual void LoadConfig(Char_t const *cname="Config");
    virtual Bool_t DoPlotTempo(void){return kFALSE;}
    virtual void PlotTempo(void){;}
    virtual TList *GetListOfParams(void) const {return listOfParams;}
    virtual void ResetListOfParams(void);
    virtual TList *GetListOfHistos(void) const {return listOfHistos;}
    virtual void ResetListOfHistos(void) {listOfHistos->RemoveAll();}
    virtual void AddHisto(TH1 *h);
    virtual void RemoveHisto(TH1 *h);
    virtual void RemoveHisto(Char_t *s);
    virtual void ClearHisto(TH1 *h);
    virtual void ClearHisto(Char_t *s);
    virtual void AddParam(FParam *p);
    virtual void RemoveParam(FParam *p);
    virtual void RemoveParam(Char_t *s);
    virtual void ClearSpectra(void);
    virtual FParam *GetParam(Char_t const *s)
    {return (FParam*)listOfParams->FindObject(s);}
    virtual TH1 *GetHisto(Char_t const *s)
    {return (TH1 *)listOfHistos->FindObject(s);}
    virtual FHistAtt *GetHistoAtt(Char_t const *s)
    {return dynamic_cast<FHistAtt *>(GetHisto(s));}
    virtual ULong64_t GetNevt(void)const{return nevt;}   // number of events

    virtual Bool_t HasNetHistoProvider(void) {return (fnhp != 0);}
    virtual Char_t const *GetNetHistoProviderName(void) const {if(fnhp) return (Char_t const *)fnhp->GetName(); else return (Char_t const *)"none";}
    virtual void SetNetHistoProvider(Char_t const *s="caep71.in2p3.fr",Int_t port=9090);
    virtual void UnsetNetHistoProvider(void);
    virtual void SendListToDataServer(void);
    virtual FNetHistoProvider *GetNetHistoProvider(void) {return fnhp;}
    virtual void DumpEvent(ostream &fout=cout);

    virtual void Build1DHistosFromParamsBruts(Int_t nBinsX=0,
        Double_t xmin=-1.,
        Double_t xmax=-1.);

    virtual void SetSaveHistos(Bool_t save=kTRUE) {doSaveHistos=save;}
    virtual Bool_t DoSaveHistos(void) {return doSaveHistos;}

    virtual void RemoveAllUserDefParams(void);
    virtual void RemoveAllHistos(void);
    virtual void RemoveAllCanvases(void);

    virtual void SetDevice(FDevice *dev=0)
    {if(dataReader && dataReader->NeedDevice()) 
     ((FDataReaderWithDevice *)dataReader)->SetDevice(dev);}
    virtual void SetDataReader(FDataReader *s=0) {dataReader=s;}
    virtual FDataReader *GetDataReader(void) {return dataReader;}
    virtual const Char_t *GetExtraInfo(void) { if (dataReader) return dataReader->GetDataReaderInfo(); else return(const Char_t *)"";}
    virtual void ResetExtraInfo(void) { if (dataReader) dataReader->ResetDataReaderInfo();}

    virtual void Fatal(const char *method, const char *msgfmt, ...) const;
    
    ClassDef(FEventProcessor,1)

};

#endif

